require 'typhoeus'
require 'json'

module YourKeymaker
  # Public: Provides APIs for ApiKey management.
  class ApiKey
    attr_accessor :key, :user_id, :expires_at

    class RequestTimedOut < StandardError; end
    class RequestFailed < StandardError; end

    # Public: Issue ApiKey for given user_id.
    #
    # Returns newly issued ApiKey instance.
    def self.issue(user_id)
      res = Typhoeus.post("#{YourKeymaker.config.host}/api_keys",
                          body: { user_id: user_id })

      raise RequestTimedOut.new if res.timed_out?
      raise RequestFailed.new(status: res.status_message) unless res.success?

      api_key = JSON.parse(res.body)
      new_from_response(api_key, user_id: user_id)
    end

    # Public: Lookup matching user for a ApiKey
    #
    # Returns ApiKey instance with user_id attached.
    def self.lookup(key)
      res = Typhoeus.get("#{YourKeymaker.config.host}/api_keys/#{key}")

      raise RequestTimedOut.new if res.timed_out?
      raise RequestFailed.new(status: res.status_message) unless res.success?

      api_key = JSON.parse(res.body)
      new(key: key, user_id: api_key['user_id'])
    end

    # Public: List valid ApiKeys for give user_id
    #
    # Returns array of ApiKeys
    def self.of(user_id)
      res = Typhoeus.get("#{YourKeymaker.config.host}/api_keys?user_id=#{user_id}")

      raise RequestTimedOut.new if res.timed_out?
      raise RequestFailed.new(status: res.status_message) unless res.success?

      api_keys = JSON.parse(res.body)

      api_keys.map do |api_key|
        new_from_response(api_key, user_id: user_id)
      end
    end

    # Public: Revoke all ApiKeys of given user_id
    #
    # Returns boolean if request was successful.
    def self.revoke_all(user_id)
      res = Typhoeus.delete("#{YourKeymaker.config.host}/api_keys?user_id=#{user_id}")

      raise RequestTimedOut.new if res.timed_out?
      raise RequestFailed.new(status: res.status_message) unless res.success?

      res.return_code == :ok
    end

    # Public: Revoke ApiKey
    #
    # Returns boolean if request was successful.
    def revoke
      res = Typhoeus.delete("#{YourKeymaker.config.host}/api_keys/#{self.key}")

      raise RequestTimedOut.new if res.timed_out?
      raise RequestFailed.new(status: res.status_message) unless res.success?

      res.return_code == :ok
    end

    def revoke!
      revoke
    end

    def initialize(args)
      args.each do |k, v|
        instance_variable_set("@#{k}", v) unless v.nil?
      end
    end

    def as_json(options={})
      {
        key: key,
        expires_at: expires_at
      }
    end

    private

    def self.new_from_response(response, options = {})
      api_key = new(key: response['key'], expires_at: response['expires_at'])
      api_key.user_id = options[:user_id] if options[:user_id]
      api_key
    end
  end
end
